"use strict";
var fs = require("fs");
var path = require("path");
var loaderUtils = require("loader-utils");
function tsloader(contents) {
    var callback = this.async();
    var tsOutputDir = this.tsOutputDir;
    var resourcePath = this.resourcePath;
    var outputRelative = path.relative(process.cwd(), resourcePath);
    var jsFilePath = path.join(tsOutputDir, outputRelative).replace('.ts', '.js');
    var sourceMapFilePath = path.join(tsOutputDir, outputRelative).replace('.ts', '.js.map');
    var loader = this;
    this.clearDependencies();
    this.addDependency(resourcePath);
    var dependencies = this.tsDependencyGraph[resourcePath];
    if (dependencies) {
        var addDependency = this.addDependency.bind(this);
        dependencies.forEach(addDependency);
    }
    if (!callback)
        return readFilesSync();
    try {
        var _a = readFilesSync(), sourceMap = _a.sourceMap, output = _a.output;
        callback(null, output, sourceMap);
    }
    catch (error) {
        callback(error);
    }
    function readFilesSync() {
        var outputText = fs.readFileSync(jsFilePath, 'utf-8');
        var sourceMapText = fs.existsSync(sourceMapFilePath) ? fs.readFileSync(sourceMapFilePath, 'utf-8') : undefined;
        var filePath = path.normalize(resourcePath);
        var result = makeSourceMap(sourceMapText, outputText, filePath, contents);
        return result;
    }
    function makeSourceMap(sourceMapText, outputText, filePath, contents) {
        if (!sourceMapText) {
            return { output: outputText, sourceMap: undefined };
        }
        return {
            output: outputText.replace(/^\/\/# sourceMappingURL=[^\r\n]*/gm, ''),
            sourceMap: Object.assign(JSON.parse(sourceMapText), {
                sources: [loaderUtils.getRemainingRequest(loader)],
                file: filePath,
                sourcesContent: [contents]
            })
        };
    }
}
module.exports = tsloader;
//# sourceMappingURL=loader.js.map