"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var glob = require("glob");
function appendTsSuffixIfMatch(patterns, path) {
    if (patterns.length > 0) {
        for (var _i = 0, patterns_1 = patterns; _i < patterns_1.length; _i++) {
            var regexp = patterns_1[_i];
            if (path.match(regexp)) {
                return path + '.ts';
            }
        }
    }
    return path;
}
exports.appendTsSuffixIfMatch = appendTsSuffixIfMatch;
/**
 * Recursively collect all possible dependencies of passed file
 */
function collectAllDependencies(dependencyGraph, filePath, collected) {
    if (collected === void 0) { collected = {}; }
    filePath = path.resolve(filePath);
    var result = {};
    result[filePath] = true;
    collected[filePath] = true;
    var directDependencies = dependencyGraph[filePath];
    if (directDependencies) {
        directDependencies.forEach(function (dependencyFilePath) {
            if (!collected[dependencyFilePath]) {
                collectAllDependencies(dependencyGraph, dependencyFilePath, collected)
                    .forEach(function (filePath) { return result[filePath] = true; });
            }
        });
    }
    return Object.keys(result);
}
exports.collectAllDependencies = collectAllDependencies;
function stringHash(content) {
    var hash = 0;
    if (content.length == 0)
        return hash;
    for (var i = 0; i < content.length; i++) {
        var char = content.charCodeAt(i);
        hash = ((hash << 5) - hash) + char;
        hash = hash & hash; // Convert to 32bit integer
    }
    return hash;
}
function getChangedSourcesAndDepenencies(allSources, allSourceHashes, cachedResults) {
    var sources = allSources.filter(function (srcPath) { return allSourceHashes[srcPath] !== cachedResults.checkSums[srcPath]; });
    if (sources.length > 0) {
        //adding dependencies
        var additionalDependencies = sources.map(function (srcFile) { return collectAllDependencies(cachedResults.dependencyGraph, srcFile); })
            .reduce(function (a1, a2) { return a1.concat(a2); }, []);
        sources.push.apply(sources, additionalDependencies);
        sources = sources.filter(function (srcPath) { return !!srcPath; });
        sources.push.apply(sources, allSources.filter(function (srcPath) { return srcPath.endsWith('.d.ts'); }));
    }
    return sources.filter(function (srcFile) { return srcFile.endsWith('ts'); }).filter(function (srcFile, index, arr) { return arr.indexOf(srcFile) === index; });
}
exports.getChangedSourcesAndDepenencies = getChangedSourcesAndDepenencies;
function calculateSourcesCheckSums(allSources) {
    return allSources
        .map(function (srcFile) { return fs.readFileSync(srcFile, 'utf-8'); })
        .map(function (value) { return value.toString(); })
        .map(function (content, index) {
        var entry = {};
        entry[allSources[index]] = stringHash(content);
        return entry;
    }).reduce(function (entry1, entry2) { return Object.assign(entry1, entry2); });
}
exports.calculateSourcesCheckSums = calculateSourcesCheckSums;
function getFilesGlob(filesGlob) {
    if (!filesGlob) {
        return [];
    }
    return filesGlob
        .map(function (entry) { return glob.sync(entry); })
        .reduce(function (a1, a2) { return a1.concat(a2); }, []);
}
exports.getFilesGlob = getFilesGlob;
//# sourceMappingURL=utils.js.map